# Birthday Reminder 1.0
# Copyright (C) 2009, Pascal M.Vaudrevange
# pascal@vaudrevange.com
#
# based on Birthwatch by Floriano Scioscia
#
# This program is free software; you can redistribute it and/or modify it under the terms
# of the GNU General Public License as published by the Free Software Foundation;
# either version 2 of the License, or (at your option) any later version.
# This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
# without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.

import e32
import appuifw
import e32calendar
import contacts
import time
import calendar


SIS_VERSION = '0.1'

app_lock=e32.Ao_lock()

def exit_key_handler():
    app_lock.signal()

class Entry:
    max_year=2037
    now=time.time()
    offset=2*24*60*60 #need to offset by two days as the saved calendar entries are lagging by 2 days
    def __init__(self, first_name, last_name, date):
        self.content=last_name+u' '+first_name+u' '+str(time.gmtime(date).tm_year)
        self.first_name=first_name
        self.last_name=last_name
        self.full_name=last_name+u', '+first_name
        self.date=self.get_date(date)
        self.repeat = self.get_repeat()
        self.vcard = self.first_name+u' '+self.last_name+'\n'+time.strftime('%d.%B %Y',time.gmtime(date))
    def __cmp__(self, other):
        if self.content < other.content:
            return -1
        else:
            return 1
    def get_repeat(self):
        bd = time.gmtime(self.date)
        year = time.gmtime(self.now).tm_year
        start = (year, bd.tm_mon, bd.tm_mday, 0, 0, 0, 0, 1, 0)
        end = (self.max_year, bd.tm_mon, bd.tm_mday, 0, 0, 0, 0, 1, 0)
        self.last_date = float(calendar.timegm(end)+self.offset)
        repeat = {'type':'yearly_by_date', 'start':float(calendar.timegm(start)+self.offset), 'end':self.last_date, 'interval':1}
        return repeat
    def get_date(self, date):
        this_year=time.gmtime(self.now).tm_year
        return float(calendar.timegm((this_year, time.gmtime(date).tm_mon, time.gmtime(date).tm_mday, 0,0,0,1,0)))
    def __str__(self):
        return self.content

class Birth:   
    def scan_contacts(self):
        db = contacts.open()
        entries=[]
        for entry_id in db:
            contact = db[entry_id]
            tmp_date = contact.find(type='date')
            if len(tmp_date) > 0:
                first_name=contact.find(type='first_name')[0].value
                last_name=contact.find(type='last_name')[0].value
                if (first_name != u'' or last_name !=u''):
                    date=tmp_date[0].value
                    entries.append(Entry(first_name, last_name, date))
        return entries
    def add_event(self, cal_db, entry):
        anniv=cal_db.add_anniversary()
        anniv.set_time(entry.date)
        anniv.set_repeat(entry.repeat)
        anniv.content=entry.content
        anniv.commit()
    def analyse_entries(self,cal_db, entries):
        self.old_entries=[]
        self.new_entries=[]
        for entry in entries:
            if len(cal_db.find_instances(entry.date, entry.last_date, entry.content))==0:
                self.new_entries.append(entry)
            else:
                self.old_entries.append(entry)
# GUI stuff
    def old_entries_handler(self):
        entry=self.old_entries[appuifw.app.body.current()]
        self.popup.show(entry.vcard)
    def new_entries_handler(self):
        entry=self.new_entries[appuifw.app.body.current()]
        self.popup.show(entry.vcard)
    def tab_old_entries(self):
        appuifw.app.menu=self.old_menu
        list=self.get_list(self.old_entries)
        if len(list)==0:
            return appuifw.Text(u'Empty')
        else:
            return appuifw.Listbox(list,self.old_entries_handler)
    def tab_new_entries(self):
        appuifw.app.menu=self.new_menu
        list=self.get_list(self.new_entries)
        if len(list)==0:
            return appuifw.Text(u'Empty')
#            return self.write_text(u'Empty')
        else:
            return appuifw.Listbox(list, self.new_entries_handler)
    def get_list(self, entries):
        list=[]
        for entry in entries:
            list.append(entry.full_name)
        return list
    def handle_tab(self, index):
        if index==0:
            appuifw.app.body = self.tab_new_entries()
        if index==1:
            appuifw.app.body = self.tab_old_entries()
    def option_add_event(self):
        name=str(appuifw.app.body)
        if name[1:16]==u'appuifw.Listbox':
            entry=self.new_entries.pop(appuifw.app.body.current())
            self.add_event(self.cal_db, entry)
            self.old_entries.append(entry)
            self.old_entries.sort()
            appuifw.app.body = self.tab_new_entries()
    def option_remove_event(self):
        name=str(appuifw.app.body)
        if name[1:16]==u'appuifw.Listbox':
            entry=self.old_entries.pop(appuifw.app.body.current())
            self.new_entries.append(entry)
            self.new_entries.sort()
            instances=self.cal_db.find_instances(entry.date, entry.last_date, entry.content)
            if len(instances)>0:
                self.cal_db.__delitem__(instances[0]['id'])
                appuifw.app.body = self.tab_old_entries()
    def option_add_all(self):
        name=str(appuifw.app.body)
        if name[1:16]==u'appuifw.Listbox':
            for entry in self.new_entries:
                self.add_event(self.cal_db, entry)
                self.old_entries.append(entry)
                self.old_entries.sort()
                self.new_entries=[]
                appuifw.app.body = self.tab_new_entries()
    def option_add_multiple(self):
        name=str(appuifw.app.body)
        if name[1:16]==u'appuifw.Listbox':
            appuifw.note(u'Not implemented')
    def write_about(self):
        appuifw.note(u'Birthday organizer\n(c) Pascal M.Vaudrevange (pascal@vaudrevange.com)\nreleased under GPL')
    def run(self):
        # start the gui
        appuifw.app.screen='normal'
        self.popup=appuifw.InfoPopup()
        about_menu=(u'About', self.write_about)
        exit_menu=(u'Exit', exit_key_handler)
        self.new_menu=[(u'Add to calendar', self.option_add_event),(u'Add multiple entries to calendar', self.option_add_multiple), (u'Add all entries to calendar', self.option_add_all), about_menu, exit_menu]
        self.old_menu=[(u'Remove from calendar', self.option_remove_event), about_menu, exit_menu]
        appuifw.app.set_tabs([u'New Entries',u'Old Entries'], self.handle_tab)
        appuifw.app.activate_tab(0)
        appuifw.app.body = self.tab_new_entries()
        appuifw.app.exit_key_handler = exit_key_handler
        app_lock.wait()
    def __init__(self):
        # initialize database
        appuifw.app.screen='normal'
        appuifw.app.directional_pad=False
        appuifw.app.title=u'Birth'
        appuifw.note(u'scanning contacts')
        self.entries=self.scan_contacts()
        self.entries.sort()
        appuifw.note(u'scanning calendar')
        self.cal_db=e32calendar.open()
        self.old_entries=[]
        self.new_entries=[]
        self.analyse_entries(self.cal_db, self.entries)


before_screen = appuifw.app.screen
before_body = appuifw.app.body
before_title = appuifw.app.title
before_menu = appuifw.app.menu
before_exit_handler = appuifw.app.exit_key_handler


b=Birth()
b.run()


# finish
appuifw.app.set_tabs([], None)
appuifw.app.body = before_body
appuifw.app.title = before_title
appuifw.app.menu = before_menu
appuifw.app.exit_key_handler = before_exit_handler
appuifw.app.screen = before_screen
#appuifw.app.set_exit()
